# Copyright (c) 2016 - Bryan Worrell
# For license information, see the LICENSE file

from __future__ import absolute_import

import sys
import functools
import contextlib

from . import utils
from . import tracers
from . import contexts
from . import constants


def _chance(func, ratio=None):
    """Derive a naroleptic "chance" from the input function. This will be the
    number of lines in the function multiplied by some ratio.

    Args:
        func: A function.
        ratio (float): The ratio of maximum lines of code to sleep() calls.

    Returns:
        An integer value representing the maximum number of lines executed
        before a sleep() call.
    """
    ratio = ratio or 0.25
    val = int(ratio * utils.numlines(func))
    return val if val > 0 else 1


class narcoleptic(object):
    """Function decorator which injects random sleep() calls into your
    function.

    Note:
        This will not descend into descendant function calls, so only the
        decorated function will change in behavior.

    Args:
        min (int): The minimum sleep time in seconds.
        max (int): The maximum sleep tim in seconds.
        chance (int): A sleep call will be executed **AT LEAST** once every
            N (chance) lines.
    """

    def __init__(self, min=None, max=None, chance=None):
        self._min = min or constants.DEFAULT_MIN_SLEEP
        self._max = max or constants.DEFAULT_MAX_SLEEP
        self._chance = chance

    def _tracer_instance(self, func):
        """Return a RandomSleepTracer object instance.

        Args:
            func (function): Used to derive a `chance` parameter from the
                number of lines in the function.

        Returns:
            A `tracer.RandomSleepTracer` object.
        """
        return tracers.RandomSleepTracer(
            min=self._min,
            max=self._max,
            chance=self._chance or _chance(func)
        )

    def __call__(self, func):
        """Return a wrapped function which injects a tracer before and after
        execution of the input function.

        Any existing tracers will be returned prior to the returned
        function exiting.

        Args:
            func: A function.

        Returns:
            A wrapped function. Any parameters will be passed to the input
            function.
        """
        @functools.wraps(func)
        def inner(*args, **kwargs):
            old = sys.gettrace()
            sys.settrace(self._tracer_instance(func))

            try:
                val = func(*args, **kwargs)
            finally:
                sys.settrace(old)
            return val

        return inner


@contextlib.contextmanager
def narcoleptic_ctx(min=None, max=None, chance=None):
    """A context manager that injects random sleep calls into the execution
    context. **THIS DOES NOT WORK.**
    """
    tracer = tracers.RandomSleepTracer(
        min=min or constants.DEFAULT_MIN_SLEEP,
        max=max or constants.DEFAULT_MAX_SLEEP,
        chance=chance or constants.DEFAULT_CHANCE
    )

    with contexts.injected(tracer):
        yield



# Copyright (c) 2016 - Bryan Worrell
# For license information, see the LICENSE file
"""
Utility functions.
"""

from __future__ import absolute_import

import dis
import time
import random
import inspect
import logging


logger = logging.getLogger(__name__)


def classname(cls):
    """Return the class name for the input class/object.

    Args:
        cls: A class or object.
    """
    if not inspect.isclass(cls):
        cls = type(cls)
    return cls.__name__


def code(func):
    """Return the __code__ for the input function."""
    return func.__code__


def numlines(func):
    """Return the number of lines in the input function."""
    lines = tuple(dis.findlinestarts(code(func)))
    first_line = lines[0][1]
    last_line  = lines[-1][1]
    return last_line - first_line + 1


def randsleep(min, max):
    """Sleep for a number of seconds between `min` and `max`.

    Args:
        min (int): The minimum number of seconds to sleep.
        max (int): The maximum number of seconds to sleep.
    """
    interval = random.randint(min, max)
    logger.info("Sleeping for %s seconds", interval)
    time.sleep(interval)




# Copyright (c) 2016 - Bryan Worrell
# For license information, see the LICENSE file
"""
Line tracers.
"""

from __future__ import absolute_import

import logging
import inspect
import random
import itertools

from . import utils


logger = logging.getLogger(__name__)


class RandomSleepTracer(object):
    """Line tracer that injects sleep() calls.

    Args:
        min (int): The minimum amount of time to sleep.
        max (int): The maximum amount of time to sleep.
        chance (int): Execute a sleep **AT LEAST** once every N lines of code.
    """

    def __init__(self, min, max, chance):
        if chance < 1:
            raise ValueError("The chance must be >= 1. Received: %s" % chance)

        self._min = min
        self._max = max
        self._chance = chance
        self._sleep_on = random.randint(1, chance)
        self._sleep_count = itertools.count(1)
        self._descend = True

    def _reset_sleep_counts(self):
        """Reset the sleep counters."""
        self._sleep_on = random.randint(1, self._chance)
        self._sleep_count = itertools.count(1)

    def _go_to_sleep(self):
        """Execute a sleep() call for a random duration between the input
        min and max seconds.
        """
        utils.randsleep(self._min, self._max)

    def _try_sleep(self):
        """Determine if it is time to sleep. If it is, sleep and reset counters.
        Otherwise, return.
        """
        if next(self._sleep_count) == self._sleep_on:
            self._go_to_sleep()
            self._reset_sleep_counts()

    def __call__(self, frame, event, args):
        """Inspect the line being executed and attempt to sleep.

        Note:
            This function will accept a single "call" event before disabling
            the line tracer. This means it accepts the call to the wrapped
            function but no other internal function calls.
        """
        logger.debug("event: %s,  %s", event, inspect.getframeinfo(frame))

        if event == "line":
            self._try_sleep()
            return self
        elif event == "call" and self._descend:
            self._descend = False
            return self
        return None





#!/usr/bin/env python
# Copyright (c) 2015 - Bryan Worrell
# For license information, see the LICENSE file
import os
import setuptools

BASE_DIR     = os.path.dirname(os.path.abspath(__file__))
VERSION_FILE = os.path.join(BASE_DIR, "narcolepsy", 'version.py')
README_FILE  = os.path.join(BASE_DIR, "README.rst")


def normalize(version):
    return version.split()[-1].strip("\"'")


def get_version():
    with open(VERSION_FILE) as f:
        version = next(line for line in f if line.startswith("__version__"))
        return normalize(version)


with open(README_FILE) as f:
    readme = f.read()


extras_require = {
    'docs': [],
    'test': [
        "nose==1.3.0",
        "tox==1.6.1"
    ],
}

setuptools.setup(
    name='narcolepsy',
    description="Give your application a break.",
    author='Bryan Worrell',
    author_email='bworrell@notmyemail.com',
    url='https://github.com/bworrell',
    version=get_version(),
    packages=setuptools.find_packages(),
    include_package_data=True,
    extras_require=extras_require,
    long_description=readme,
    keywords="narcolepsy sleep timing useless"
)



# Copyright (c) 2016 - Bryan Worrell
# For license information, see the LICENSE file

from __future__ import absolute_import

import sys
import contextlib


@contextlib.contextmanager
def injected(tracer):
    """Inject trace object into the execution context. Any existing tracer
    will be reset before exiting the context.
    """
    old = sys.gettrace()
    try:
        sys.settrace(tracer)
        yield
    finally:
        sys.settrace(old)

